@@ -1,3 +1,4 @@
+"""Sereval loss functions and high level loss function get'er."""
 import math
 import torch
 import torch.nn as nn
@@ -7,22 +8,27 @@ import logging
 from typing import Optional
 
 from nowcasting_utils.models.losses.FocalLoss import FocalLoss
-from nowcasting_utils.models.losses.StructuralSimilarity import SSIMLoss, MS_SSIMLoss, SSIMLossDynamic
+from nowcasting_utils.models.losses.StructuralSimilarity import (
+    SSIMLoss,
+    MS_SSIMLoss,
+    SSIMLossDynamic,
+)
 from nowcasting_utils.models.losses.TotalVariationLoss import TVLoss
 
 logger = logging.getLogger(__name__)
 
 
 class WeightedLosses:
+    """Class: Weighted loss depending on the forecast horizon."""
+
     def __init__(self, decay_rate: Optional[int] = None, forecast_length: int = 6):
         """
-         Want to set up the MSE loss function so the weights only have to be calculated once.
+        Want to set up the MSE loss function so the weights only have to be calculated once.
 
         Args:
             decay_rate: The weights exponentially decay depending on the 'decay_rate'.
             forecast_length: The forecast length is needed to make sure the weights sum to 1
         """
-
         self.decay_rate = decay_rate
         self.forecast_length = forecast_length
 
@@ -47,7 +53,7 @@ class WeightedLosses:
         self.weights = self.weights.to(device)
 
     def get_mse_exp(self, output, target):
-        """Loss function weighted MSE """
+        """Loss function weighted MSE"""
         return torch.sum(self.weights * (output - target) ** 2)
 
     def get_mae_exp(self, output, target):
@@ -57,14 +63,30 @@ class WeightedLosses:
 
 class GradientDifferenceLoss(nn.Module):
     """
-    Gradient Difference Loss that penalizes blurry images more than MSE
+    Gradient Difference Loss that penalizes blurry images more than MSE.
     """
 
     def __init__(self, alpha: int = 2):
+        """
+        Initalize the Loss Class.
+
+        Args:
+            alpha: #TODO
+        """
         super(GradientDifferenceLoss, self).__init__()
         self.alpha = alpha
 
     def forward(self, x: torch.Tensor, y: torch.Tensor):
+        """
+        Calculate the Gradient Difference Loss.
+
+        Args:
+            x: vector one
+            y: vector two
+
+        Returns: the Gradient Difference Loss value
+
+        """
         t1 = torch.pow(
             torch.abs(
                 torch.abs(x[:, :, :, 1:, :] - x[:, :, :, :-1, :])
@@ -80,26 +102,32 @@ class GradientDifferenceLoss(nn.Module):
             self.alpha,
         )
         # Pad out the last dim in each direction so shapes match
-        t1 = F.pad(input=t1, pad=(0, 0, 1, 0), mode='constant', value=0)
-        t2 = F.pad(input=t2, pad=(0, 1, 0, 0), mode='constant', value=0)
+        t1 = F.pad(input=t1, pad=(0, 0, 1, 0), mode="constant", value=0)
+        t2 = F.pad(input=t2, pad=(0, 1, 0, 0), mode="constant", value=0)
         loss = t1 + t2
         return loss.mean()
 
 
 class GridCellLoss(nn.Module):
-    """
-    Grid Cell Regularizer loss from Skillful Nowcasting, see https://arxiv.org/pdf/2104.00954.pdf
-
-    """
+    """Grid Cell Regularizer loss from Skillful Nowcasting, see https://arxiv.org/pdf/2104.00954.pdf."""
 
     def __init__(self, weight_fn=None):
+        """
+        Initialize the model.
+
+        Args:
+            weight_fn: the weight function the be called when #TODO?
+        """
         super().__init__()
         self.weight_fn = weight_fn  # In Paper, weight_fn is max(y+1,24)
 
     def forward(self, generated_images, targets):
         """
-        Calculates the grid cell regularizer value, assumes generated images are the mean predictions from
+        Calculates the grid cell regularizer value.
+
+        This assumes generated images are the mean predictions from
         6 calls to the generater (Monte Carlo estimation of the expectations for the latent variable)
+
         Args:
             generated_images: Mean generated images from the generator
             targets: Ground truth future frames
@@ -116,13 +144,24 @@ class GridCellLoss(nn.Module):
 
 class NowcastingLoss(nn.Module):
     """
-    Loss described in Skillful-Nowcasting GAN,  see https://arxiv.org/pdf/2104.00954.pdf
+    Loss described in Skillful-Nowcasting GAN,  see https://arxiv.org/pdf/2104.00954.pdf.
     """
 
     def __init__(self):
+        """Initialize the model."""
         super().__init__()
 
     def forward(self, x, real_flag):
+        """
+        Forward step.
+
+        Args:
+            x: the data to work with
+            real_flag: boolean if its real or not
+
+        Returns: #TODO
+
+        """
         if real_flag is True:
             x = -x
         return F.relu(1.0 + x).mean()
@@ -130,8 +169,7 @@ class NowcastingLoss(nn.Module):
 
 def get_loss(loss: str = "mse", **kwargs) -> torch.nn.Module:
     """
-
-    Function to get different losses easily
+    Function to get different losses easily.
 
     Args:
         loss: name of the loss, or torch.nn.Module, if a Module, returns that Module
@@ -174,7 +212,11 @@ def get_loss(loss: str = "mse", **kwargs) -> torch.nn.Module:
     elif loss in ["l1"]:
         criterion = torch.nn.L1Loss()
     elif loss in ["tv", "total_variation"]:
-        criterion = TVLoss(tv_type=kwargs.get("tv_type", "tv"), p=kwargs.get("p", 1), reduction=kwargs.get("reduction", "mean"))
+        criterion = TVLoss(
+            tv_type=kwargs.get("tv_type", "tv"),
+            p=kwargs.get("p", 1),
+            reduction=kwargs.get("reduction", "mean"),
+        )
     elif loss in ["gdl", "gradient_difference_loss"]:
         criterion = GradientDifferenceLoss(alpha=kwargs.get("alpha", 2))
     elif loss in ["weighted_mse", "weighted_mae"]:
